=== modified file 'drizzled/message/Makefile.am'
--- drizzled/message/Makefile.am	2009-04-25 19:44:53 +0000
+++ drizzled/message/Makefile.am	2009-06-03 20:31:33 +0000
@@ -27,6 +27,7 @@
 		table_reader \
 		table_writer \
 		transaction_reader \
+		command_reader \
 		transaction_writer \
 		master_list_reader \
 		master_list_writer \
@@ -77,6 +78,8 @@
 
 transaction_reader_SOURCES =  transaction_reader.cc
 
+command_reader_SOURCES =  command_reader.cc
+
 length_SOURCES = length.cc
 
 EXTRA_DIST = \

=== added file 'drizzled/message/command_reader.cc'
--- drizzled/message/command_reader.cc	1970-01-01 00:00:00 +0000
+++ drizzled/message/command_reader.cc	2009-06-03 20:31:33 +0000
@@ -0,0 +1,244 @@
+#include <drizzled/global.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <iostream>
+#include <fstream>
+#include <string>
+#include <unistd.h>
+#include <drizzled/message/transaction.pb.h>
+
+using namespace std;
+using namespace drizzled::message;
+
+/**
+ * @file Example application for reading change records (Command messages)
+ *
+ * @note
+ *
+ * This program is used in the serial_event_log test suite to verify
+ * the log written by that plugin.
+ */
+
+void printInsert(const drizzled::message::Command &container, const drizzled::message::InsertRecord &record)
+{
+
+  cout << "INSERT INTO `" << container.schema() << "`.`" << container.table() << "` (";
+
+  int32_t num_fields= record.insert_field_size();
+
+  int32_t x;
+  for (x= 0; x < num_fields; x++)
+  {
+    if (x != 0)
+      cout << ", ";
+
+    const Table::Field f= record.insert_field(x);
+
+    cout << "`" << f.name() << "`";
+  }
+
+  cout << ") VALUES ";
+
+  /* 
+   * There may be an INSERT VALUES (),() type statement.  We know the
+   * number of records is equal to the field_values array size divided
+   * by the number of fields.
+   *
+   * So, we do an inner and an outer loop.  Outer loop is on the number
+   * of records and the inner loop on the number of fields.  In this way, 
+   * we know that record.field_values(outer_loop * num_fields) + inner_loop))
+   * always gives us our correct field value.
+   */
+  int32_t num_records= (record.insert_value_size() / num_fields);
+  int32_t y;
+  for (x= 0; x < num_records; x++)
+  {
+    if (x != 0)
+      cout << ", ";
+
+    cout << "(";
+    for (y= 0; y < num_fields; y++)
+    {
+      if (y != 0)
+        cout << ", ";
+
+      cout << "\"" << record.insert_value((x * num_fields) + y) << "\"";
+    }
+    cout << ")";
+  }
+
+  cout << ";";
+}
+
+void printDeleteWithPK(const drizzled::message::Command &container, const drizzled::message::DeleteRecord &record)
+{
+  cout << "DELETE FROM `" << container.schema() << "`.`" << container.table() << "`";
+
+  int32_t num_where_fields= record.where_field_size();
+  /* 
+   * Make sure we catch anywhere we're not aligning the fields with
+   * the field_values arrays...
+   */
+  assert(num_where_fields == record.where_value_size());
+
+  cout << " WHERE ";
+  int32_t x;
+  for (x= 0; x < num_where_fields; x++)
+  {
+    if (x != 0)
+      cout << " AND "; /* Always AND condition with a multi-column PK */
+
+    const Table::Field f= record.where_field(x);
+
+    /* Always equality conditions */
+    cout << "`" << f.name() << "` = \"" << record.where_value(x) << "\"";
+  }
+}
+
+void printUpdateWithPK(const drizzled::message::Command &container, const drizzled::message::UpdateRecord &record)
+{
+  int32_t num_update_fields= record.update_field_size();
+  int32_t x;
+
+  cout << "UPDATE `" << container.schema() << "`.`" << container.table() << "` SET ";
+
+  for (x= 0;x < num_update_fields; x++)
+  {
+    Table::Field f= record.update_field(x);
+    
+    if (x != 0)
+      cout << ", ";
+
+    cout << "`" << f.name() << "` = \"" << record.after_value(x) << "\"";
+  }
+
+  int32_t num_where_fields= record.where_field_size();
+  /* 
+   * Make sure we catch anywhere we're not aligning the fields with
+   * the field_values arrays...
+   */
+  assert(num_where_fields == record.where_value_size());
+
+  cout << " WHERE ";
+  for (x= 0;x < num_where_fields; x++)
+  {
+    if (x != 0)
+      cout << " AND "; /* Always AND condition with a multi-column PK */
+
+    const Table::Field f= record.where_field(x);
+
+    /* Always equality conditions */
+    cout << "`" << f.name() << "` = \"" << record.where_value(x) << "\"";
+  }
+}
+
+void printCommand(const drizzled::message::Command &command)
+{
+  cout << "/* Timestamp: " << command.timestamp() << " */"<< endl;
+
+  drizzled::message::TransactionContext trx= command.transaction_context();
+
+  cout << "/* SID: " << trx.server_id() << " XID: " << trx.transaction_id() << " */ ";
+
+  switch (command.type())
+  {
+    case Command::START_TRANSACTION:
+      cout << "START TRANSACTION;";
+      break;
+    case Command::COMMIT:
+      cout << "COMMIT;";
+      break;
+    case Command::ROLLBACK:
+      cout << "ROLLBACK;";
+      break;
+    case Command::INSERT:
+    {
+      printInsert(command, command.insert_record());
+      break;
+    }
+    case Command::DELETE:
+    {
+      printDeleteWithPK(command, command.delete_record());
+      break;
+    }
+    case Command::UPDATE:
+    {
+      printUpdateWithPK(command, command.update_record());
+      break;
+    }
+    case Command::RAW_SQL:
+    {
+      std::string sql= command.sql();
+      /* Replace \n with spaces */
+      const std::string newline= "\n";
+      while (sql.find(newline) != std::string::npos)
+        sql.replace(sql.find(newline), 1, " ");
+
+      cout << sql << ";";
+      break;
+    }
+    default:
+      cout << "Received an unknown Command type: " << (int32_t) command.type();
+  }
+  cout << endl;
+}
+
+int main(int argc, char* argv[])
+{
+  GOOGLE_PROTOBUF_VERIFY_VERSION;
+  int file;
+
+  if (argc != 2)
+  {
+    cerr << "Usage:  " << argv[0] << " TRANSACTION_LOG" << endl;
+    return -1;
+  }
+
+  Command command;
+
+  if ((file= open(argv[1], O_RDONLY)) == -1)
+  {
+    cerr << "Can not open file: " << argv[1] << endl;
+  }
+
+  char *buffer= NULL;
+  char *temp_buffer;
+
+  while (1)
+  {
+    uint64_t length;
+
+    /* Read the size */
+    if (read(file, &length, sizeof(uint64_t)) != sizeof(uint64_t))
+      break;
+
+    if (length > SIZE_MAX)
+    {
+      cerr << "Attempted to read record bigger than SIZE_MAX" << endl;
+      exit(1);
+    }
+
+    temp_buffer= (char *)realloc(buffer, (size_t)length);
+    if (temp_buffer == NULL)
+    {
+      cerr << "Memory allocation failure trying to allocate " << length << " bytes."  << endl;
+      exit(1);
+    }
+    memset(temp_buffer, 0, (size_t)length);
+    buffer= temp_buffer;
+    size_t read_bytes= 0;
+
+    /* Read the transaction */
+    if ((read_bytes= read(file, buffer, (size_t)length)) != (size_t)length)
+    {
+      cerr << "Could not read entire transaction. Read " << read_bytes << " bytes instead of " << length << " bytes." << endl;
+      exit(1);
+    }
+    command.ParseFromArray(buffer, (int) length);
+
+    /* Print the command */
+    printCommand(command);
+  }
+  return 0;
+}

=== modified file 'drizzled/plugin_registry.cc'
--- drizzled/plugin_registry.cc	2009-04-13 16:22:40 +0000
+++ drizzled/plugin_registry.cc	2009-06-03 20:31:33 +0000
@@ -134,9 +134,14 @@
   add_protocol_factory(factory);
 }
 
-void PluginRegistry::add(drizzled::plugin::Replicator *repl)
-{
-  add_replicator(repl);
+void PluginRegistry::add(drizzled::plugin::Replicator *replicator)
+{
+  add_replicator(replicator);
+}
+
+void PluginRegistry::add(drizzled::plugin::Applier *applier)
+{
+  add_applier(applier);
 }
 
 void PluginRegistry::remove(StorageEngine *engine)
@@ -184,7 +189,12 @@
   remove_protocol_factory(factory);
 }
 
-void PluginRegistry::remove(drizzled::plugin::Replicator *repl)
-{
-  remove_replicator(repl);
+void PluginRegistry::remove(drizzled::plugin::Replicator *replicator)
+{
+  remove_replicator(replicator);
+}
+
+void PluginRegistry::remove(drizzled::plugin::Applier *applier)
+{
+  remove_applier(applier);
 }

=== modified file 'drizzled/plugin_registry.h'
--- drizzled/plugin_registry.h	2009-04-12 09:10:36 +0000
+++ drizzled/plugin_registry.h	2009-06-03 20:31:33 +0000
@@ -40,6 +40,7 @@
 namespace plugin
 {
 class Replicator;
+class Applier;
 }
 }
 
@@ -70,7 +71,8 @@
   void add(QueryCache *qcache);
   void add(SchedulerFactory *scheduler);
   void add(ProtocolFactory *protocol);
-  void add(drizzled::plugin::Replicator *repl);
+  void add(drizzled::plugin::Replicator *replicator);
+  void add(drizzled::plugin::Applier *applier);
 
   void remove(StorageEngine *engine);
   void remove(InfoSchemaTable *schema_table);
@@ -81,7 +83,8 @@
   void remove(QueryCache *qcache);
   void remove(SchedulerFactory *scheduler);
   void remove(ProtocolFactory *protocol);
-  void remove(drizzled::plugin::Replicator *repl);
+  void remove(drizzled::plugin::Replicator *replicator);
+  void remove(drizzled::plugin::Applier *applier);
 
 };
 

=== modified file 'drizzled/temporal.h'
--- drizzled/temporal.h	2009-03-05 07:15:28 +0000
+++ drizzled/temporal.h	2009-06-03 20:31:33 +0000
@@ -1,11 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems
- *
- *  Authors:
- *
- *  Jay Pipes <jay.pipes@sun.com>
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by

=== modified file 'drizzled/transaction_services.cc'
--- drizzled/transaction_services.cc	2009-04-10 22:18:10 +0000
+++ drizzled/transaction_services.cc	2009-06-03 20:31:33 +0000
@@ -55,16 +55,25 @@
 
 drizzled::TransactionServices transaction_services;
 
-void add_replicator(drizzled::plugin::Replicator *repl)
-{
-  transaction_services.attachReplicator(repl);
-}
-
-void remove_replicator(drizzled::plugin::Replicator *repl)
-{
-  transaction_services.detachReplicator(repl);
-}
-
+void add_replicator(drizzled::plugin::Replicator *replicator)
+{
+  transaction_services.attachReplicator(replicator);
+}
+
+void remove_replicator(drizzled::plugin::Replicator *replicator)
+{
+  transaction_services.detachReplicator(replicator);
+}
+
+void add_applier(drizzled::plugin::Applier *applier)
+{
+  transaction_services.attachApplier(applier);
+}
+
+void remove_applier(drizzled::plugin::Applier *applier)
+{
+  transaction_services.detachApplier(applier);
+}
 
 namespace drizzled
 {

=== modified file 'drizzled/transaction_services.h'
--- drizzled/transaction_services.h	2009-04-10 22:18:10 +0000
+++ drizzled/transaction_services.h	2009-06-03 20:31:33 +0000
@@ -40,8 +40,11 @@
   }
 }
 
-void add_replicator(drizzled::plugin::Replicator *repl);
-void remove_replicator(drizzled::plugin::Replicator *repl);
+void add_replicator(drizzled::plugin::Replicator *replicator);
+void remove_replicator(drizzled::plugin::Replicator *replicator);
+
+void add_applier(drizzled::plugin::Applier *applier);
+void remove_applier(drizzled::plugin::Applier *applier);
 
 /**
  * This is a class which manages transforming internal 

=== added directory 'plugin/default_replicator'
=== added file 'plugin/default_replicator/AUTHORS'
--- plugin/default_replicator/AUTHORS	1970-01-01 00:00:00 +0000
+++ plugin/default_replicator/AUTHORS	2009-06-03 20:31:33 +0000
@@ -0,0 +1,2 @@
+Jay Pipes <joinfu@sun.com>
+

=== added file 'plugin/default_replicator/default_replicator.cc'
--- plugin/default_replicator/default_replicator.cc	1970-01-01 00:00:00 +0000
+++ plugin/default_replicator/default_replicator.cc	2009-06-03 20:31:33 +0000
@@ -0,0 +1,113 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the implementation of the default replicator.
+ *
+ * @see drizzled/plugin/replicator.h
+ * @see drizzled/plugin/applier.h
+ *
+ * @details
+ *
+ * This is a very simple implementation.  All we do is pass along the 
+ * event to the supplier.  This is meant as a skeleton replicator only.
+ *
+ * @todo
+ *
+ * Want a neat project?  Take this skeleton replicator and make a
+ * simple filtered replicator which allows the user to filter out
+ * events based on a schema or table name...
+ */
+
+#include "default_replicator.h"
+
+#include <drizzled/gettext.h>
+#include <drizzled/message/transaction.pb.h>
+
+#include <vector>
+#include <string>
+
+using namespace std;
+
+static bool sysvar_default_replicator_enable= false;
+
+bool DefaultReplicator::isActive()
+{
+  return sysvar_default_replicator_enable;
+}
+
+void DefaultReplicator::replicate(drizzled::plugin::Applier *in_applier, drizzled::message::Command *to_replicate)
+{
+  /* 
+   * We do absolutely nothing but call the applier's apply() method, passing
+   * along the supplied Command.  Yep, told you it was simple...
+   */
+  in_applier->apply(to_replicate);
+}
+
+static DefaultReplicator *default_replicator= NULL; /* The singleton replicator */
+
+static int init(PluginRegistry &registry)
+{
+  default_replicator= new DefaultReplicator();
+  registry.add(default_replicator);
+  return 0;
+}
+
+static int deinit(PluginRegistry &registry)
+{
+  if (default_replicator)
+  {
+    registry.remove(default_replicator);
+    delete default_replicator;
+  }
+  return 0;
+}
+
+static DRIZZLE_SYSVAR_BOOL(
+  enable,
+  sysvar_default_replicator_enable,
+  PLUGIN_VAR_NOCMDARG,
+  N_("Enable default replicator"),
+  NULL, /* check func */
+  NULL, /* update func */
+  false /* default */);
+
+static struct st_mysql_sys_var* default_replicator_system_variables[]= {
+  DRIZZLE_SYSVAR(enable),
+  NULL
+};
+
+drizzle_declare_plugin(default_replicator)
+{
+  "default_replicator",
+  "0.1",
+  "Jay Pipes",
+  N_("Default Replicator"),
+  PLUGIN_LICENSE_GPL,
+  init, /* Plugin Init */
+  deinit, /* Plugin Deinit */
+  NULL, /* status variables */
+  default_replicator_system_variables, /* system variables */
+  NULL    /* config options */
+}
+drizzle_declare_plugin_end;

=== added file 'plugin/default_replicator/default_replicator.h'
--- plugin/default_replicator/default_replicator.h	1970-01-01 00:00:00 +0000
+++ plugin/default_replicator/default_replicator.h	2009-06-03 20:31:33 +0000
@@ -0,0 +1,79 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the API of the default serial event log.
+ *
+ * @see drizzled/plugin/replicator.h
+ * @see drizzled/plugin/applier.h
+ *
+ * @details
+ *
+ * The SerialEventLog applies events it receives from the TransactionServices
+ * server component to a simple log file on disk.
+ * 
+ * Events are received in no guaranteed order and the serial event log
+ * is in charge of writing these events to the log as they are received.
+ */
+
+#ifndef DRIZZLE_PLUGIN_SERIAL_EVENT_LOG_H
+#define DRIZZLE_PLUGIN_SERIAL_EVENT_LOG_H
+
+#include <drizzled/server_includes.h>
+#include <drizzled/atomics.h>
+#include <drizzled/plugin/replicator.h>
+#include <drizzled/plugin/applier.h>
+
+#include <vector>
+#include <string>
+
+class DefaultReplicator: public drizzled::plugin::Replicator
+{
+public:
+  DefaultReplicator() {}
+
+  /** Destructor */
+  ~DefaultReplicator() {}
+  /**
+   * Replicate a Command message to an Applier.
+   *
+   * @note
+   *
+   * It is important to note that memory allocation for the 
+   * supplied pointer is not guaranteed after the completion 
+   * of this function -- meaning the caller can dispose of the
+   * supplied message.  Therefore, replicators and appliers 
+   * implementing an asynchronous replication system must copy
+   * the supplied message to their own controlled memory storage
+   * area.
+   *
+   * @param Command message to be replicated
+   */
+  void replicate(drizzled::plugin::Applier *in_applier, drizzled::message::Command *to_replicate);
+  
+  /** 
+   * Returns whether the default replicator is active.
+   */
+  bool isActive();
+};
+
+#endif /* DRIZZLE_PLUGIN_SERIAL_EVENT_LOG_H */

=== added file 'plugin/default_replicator/plugin.ini'
--- plugin/default_replicator/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/default_replicator/plugin.ini	2009-06-03 20:31:33 +0000
@@ -0,0 +1,7 @@
+[plugin]
+name=default_replicator
+title=Default Replicator
+description=A simple default replicator which replicates all write events to all appliers
+load_by_default=yes
+sources=default_replicator.cc
+headers=default_replicator.h

=== added directory 'plugin/serial_event_log'
=== added file 'plugin/serial_event_log/AUTHORS'
--- plugin/serial_event_log/AUTHORS	1970-01-01 00:00:00 +0000
+++ plugin/serial_event_log/AUTHORS	2009-06-03 20:31:33 +0000
@@ -0,0 +1,2 @@
+Jay Pipes <joinfu@sun.com>
+

=== added file 'plugin/serial_event_log/plugin.ini'
--- plugin/serial_event_log/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/serial_event_log/plugin.ini	2009-06-03 20:31:33 +0000
@@ -0,0 +1,7 @@
+[plugin]
+name=serial_event_log
+title=Serial Event Log
+description=Default Serial Event Log for Drizzle
+load_by_default=yes
+sources=serial_event_log.cc
+headers=serial_event_log.h

=== added file 'plugin/serial_event_log/serial_event_log.cc'
--- plugin/serial_event_log/serial_event_log.cc	1970-01-01 00:00:00 +0000
+++ plugin/serial_event_log/serial_event_log.cc	2009-06-03 20:31:33 +0000
@@ -0,0 +1,198 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the implementation of the default serial event log.
+ *
+ * @see drizzled/plugin/replicator.h
+ * @see drizzled/plugin/applier.h
+ *
+ * @details
+ *
+ * Currently, this is a very simple implementation.  We have a global
+ * lock on the file writer and write the events as the are received, 
+ * first writing a 64-bit length and then the serialized transaction/command.
+ *
+ * @todo
+ *
+ * Possibly look at a scoreboard approach with multiple file segments.  For
+ * right now, though, this is just a quick simple implementation to serve
+ * as a skeleton and a springboard.
+ */
+
+#include "serial_event_log.h"
+
+#include <drizzled/gettext.h>
+#include <drizzled/message/transaction.pb.h>
+
+#include <vector>
+#include <string>
+
+using namespace std;
+
+/** Serial Event Log plugin system variable - The path to the log file used */
+static char* sysvar_serial_event_log_file= NULL;
+static const char DEFAULT_LOG_FILE_PATH[16]= "event.log"; /* In datadir... */
+
+SerialEventLog::SerialEventLog(const char *in_log_file_path)
+  : 
+    drizzled::plugin::Applier(),
+    state(SerialEventLog::OFFLINE),
+    is_active(false),
+    log_file_path(in_log_file_path)
+{
+  /* Setup our lock protection and our log file... */
+  if (pthread_mutex_init(&lock, NULL) != 0)
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, _("Failed to initialize lock on serial event log.  Got error: %s"), strerror(errno));
+    is_active= false;
+    return;
+  }
+
+  log_file= open(log_file_path, O_APPEND|O_CREAT|O_SYNC|O_WRONLY, S_IRWXU);
+  if (log_file == -1)
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, _("Failed to open serial event log file.  Got error: %s"), strerror(errno));
+    is_active= false;
+    return;
+  }
+
+  state= SerialEventLog::ONLINE;
+  is_active= true;
+}
+
+SerialEventLog::~SerialEventLog()
+{
+  /* Clear up any resources we've consumed */
+  if (is_active && state != SerialEventLog::CRASHED && log_file != -1)
+  {
+    (void) close(log_file);
+  }
+  if (pthread_mutex_destroy(&lock) != 0)
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, _("Failed to destroy lock on serial event log.  Got error: %s"), strerror(errno));
+  }
+}
+
+bool SerialEventLog::isActive()
+{
+  return is_active;
+}
+
+void SerialEventLog::apply(drizzled::message::Command *to_apply)
+{
+  std::string buffer; /* Buffer we will write serialized command to */
+  size_t length;
+  size_t written;
+
+  to_apply->SerializeToString(&buffer);
+  length= buffer.length();
+
+  pthread_mutex_lock(&lock);
+
+  /* 
+   * Quick safety...if an error occurs below, the log file will
+   * not be active, therefore a caller could have been waiting 
+   * to write...
+   */
+  if (unlikely(state == SerialEventLog::CRASHED))
+  {
+    pthread_mutex_unlock(&lock);
+    return;
+  }
+
+  written= write(log_file, &length, sizeof(uint64_t));
+  if (unlikely(written != sizeof(uint64_t)))
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, 
+                  _("Failed to write full size of command.  Tried to write %" PRId64 ", but only wrote %" PRId64 ".  Error: %s"), 
+                  (uint64_t) length, 
+                  (uint64_t) written, 
+                  strerror(errno));
+    state= CRASHED;
+    is_active= false;
+    pthread_mutex_unlock(&lock);
+    return;
+  }
+
+  written= write(log_file, buffer.c_str(), length);
+  if (unlikely(written != length))
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, 
+                  _("Failed to write full serialized command.  Tried to write %" PRId64 ", but only wrote %" PRId64 ".  Error: %s"), 
+                  (uint64_t) length, 
+                  (uint64_t) written, 
+                  strerror(errno));
+    state= CRASHED;
+    is_active= false;
+    pthread_mutex_unlock(&lock);
+    return;
+  }
+  pthread_mutex_unlock(&lock);
+}
+
+static SerialEventLog *serial_event_log= NULL; /* The singleton serial log */
+
+static int init(PluginRegistry &registry)
+{
+  serial_event_log= new SerialEventLog(sysvar_serial_event_log_file);
+  registry.add(serial_event_log);
+  return 0;
+}
+
+static int deinit(PluginRegistry &registry)
+{
+  if (serial_event_log)
+  {
+    registry.remove(serial_event_log);
+    delete serial_event_log;
+  }
+  return 0;
+}
+
+static DRIZZLE_SYSVAR_STR(log_file,
+                          sysvar_serial_event_log_file,
+                          PLUGIN_VAR_READONLY,
+                          N_("Path to the file to use for serial event log."),
+                          NULL, /* check func */
+                          NULL, /* update func*/
+                          DEFAULT_LOG_FILE_PATH /* default */);
+
+static struct st_mysql_sys_var* system_variables[]= {
+  DRIZZLE_SYSVAR(log_file),
+  NULL
+};
+
+drizzle_declare_plugin(serial_event_log)
+{
+  "serial_event_log",
+  "0.1",
+  "Jay Pipes",
+  N_("Default Serial Event Log"),
+  PLUGIN_LICENSE_GPL,
+  init, /* Plugin Init */
+  deinit, /* Plugin Deinit */
+  NULL, /* status variables */
+  system_variables, /* system variables */
+  NULL    /* config options */
+}
+drizzle_declare_plugin_end;

=== added file 'plugin/serial_event_log/serial_event_log.h'
--- plugin/serial_event_log/serial_event_log.h	1970-01-01 00:00:00 +0000
+++ plugin/serial_event_log/serial_event_log.h	2009-06-03 20:31:33 +0000
@@ -0,0 +1,102 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the API of the default serial event log.
+ *
+ * @see drizzled/plugin/replicator.h
+ * @see drizzled/plugin/applier.h
+ *
+ * @details
+ *
+ * The SerialEventLog applies events it receives from the TransactionServices
+ * server component to a simple log file on disk.
+ * 
+ * Events are received in no guaranteed order and the serial event log
+ * is in charge of writing these events to the log as they are received.
+ */
+
+#ifndef DRIZZLE_PLUGIN_SERIAL_EVENT_LOG_H
+#define DRIZZLE_PLUGIN_SERIAL_EVENT_LOG_H
+
+#include <drizzled/server_includes.h>
+#include <drizzled/atomics.h>
+#include <drizzled/plugin/replicator.h>
+#include <drizzled/plugin/applier.h>
+
+#include <vector>
+#include <string>
+
+class SerialEventLog: public drizzled::plugin::Applier 
+{
+public:
+  enum status
+  {
+    CRASHED= -1,
+    OFFLINE= 0, /* Default state, uninited. */
+    ONLINE,
+    WRITING
+  };
+private:
+  int log_file; /**< Handle for our log file */
+  pthread_mutex_t lock; /**< Lock for entire log */
+  enum status state; /**< The state the log is in */
+  bool is_active; /**< Internal toggle. If true, log was initialized properly... */
+  const char *log_file_path;
+public:
+  SerialEventLog(const char *in_log_file_path);
+
+  /** Destructor */
+  ~SerialEventLog();
+
+  /**
+   * Applies a Command to the serial log
+   *
+   * @note
+   *
+   * It is important to note that memory allocation for the 
+   * supplied pointer is not guaranteed after the completion 
+   * of this function -- meaning the caller can dispose of the
+   * supplied message.  Therefore, appliers which are
+   * implementing an asynchronous replication system must copy
+   * the supplied message to their own controlled memory storage
+   * area.
+   *
+   * @param Command message to be replicated
+   */
+  void apply(drizzled::message::Command *to_apply);
+  
+  /** 
+   * Returns whether the serial event log is active.
+   */
+  bool isActive();
+
+  /**
+   * Returns the state that the log is in
+   */
+  inline enum status getState()
+  {
+    return state;
+  }
+};
+
+#endif /* DRIZZLE_PLUGIN_SERIAL_EVENT_LOG_H */

=== added directory 'tests/suite/serial_event_log'
=== added directory 'tests/suite/serial_event_log/r'
=== added file 'tests/suite/serial_event_log/r/simple.result'
--- tests/suite/serial_event_log/r/simple.result	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/r/simple.result	2009-06-03 20:31:33 +0000
@@ -0,0 +1,15 @@
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+INSERT INTO t1 VALUES (1, "I love testing.");
+INSERT INTO t1 VALUES (2, "I hate testing.");
+/*  */
+/* SID: 1 XID: 0 */ DROP TABLE IF EXISTS t1;
+/*  */
+/* SID: 1 XID: 0 */ CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+/*  */
+/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `padding`) VALUES ("1", "I love testing.");
+/*  */
+/* SID: 1 XID: 4 */ INSERT INTO `test`.`t1` (`id`, `padding`) VALUES ("2", "I hate testing.");

=== added directory 'tests/suite/serial_event_log/t'
=== added file 'tests/suite/serial_event_log/t/simple-master.opt'
--- tests/suite/serial_event_log/t/simple-master.opt	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/t/simple-master.opt	2009-06-03 20:31:33 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable

=== added file 'tests/suite/serial_event_log/t/simple.test'
--- tests/suite/serial_event_log/t/simple.test	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/t/simple.test	2009-06-03 20:31:33 +0000
@@ -0,0 +1,24 @@
+# 
+# Simple test of the serial event log
+# 
+# We create a table and insert some records
+# into it.
+# 
+# We then use the transaction_reader in drizzled/message/ to read the events.
+#
+
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+
+CREATE TABLE t1 (
+  id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+
+INSERT INTO t1 VALUES (1, "I love testing.");
+INSERT INTO t1 VALUES (2, "I hate testing.");
+
+# Read in the event.log.  We must cut out all the Timestamp: XXXXXX
+# because they are not deterministic.
+--exec ../drizzled/message/command_reader var/master-data/event.log | sed 's/Timestamp: [0-9]*//g'

